#!/usr/bin/env php
<?php
/**
 * Create a new messages.POT file and update specified .po files.
 *
 * Uses find, xargs, sed, xgettext and msgmerge tools.
 * This only works on UNIXoid systems.
 *
 * This file is part of the Quam Plures project - {@link http://quamplures.net/}.
 * See also {@link https://launchpad.net/quam-plures}.
 *
 * @copyright (c) 2009-2010 by the Quam Plures developers - {@link http://quamplures.net/}
 * @copyright (c)2003-2007 by Francois PLANQUE - {@link http://fplanque.net/}.
 * Parts of this file are copyright (c)2004 by Daniel HAHLER - {@link http://thequod.de/contact}.
 *
 * @license http://quamplures.net/license.html GNU General Public License (GPL)
 *
 * {@internal Open Source relicensing agreement:
 * Daniel HAHLER grants Francois PLANQUE the right to license
 * Daniel HAHLER's contributions to this file and the b2evolution project
 * under any OSI approved OSS license (http://www.opensource.org/licenses/).
 * }
 *
 * @package internal
 *
 * @todo Add checks for format, headers and domain ("msgfmt -c").
 *
 * {@internal Below is a list of authors who have contributed to design/coding of this file:}
 * @author blueyed: Daniel HAHLER.
 * @author Tblue: Tilman BLUMENBACH.
 */

// Cache script base name:
$my_basename = basename( $argv[0] );
// Set default application root directory:
$_app_root = dirname( __FILE__ ).'/../../trunk';

function echo_usage()
{
	global $argv, $app_name, $my_basename, $_app_root, $locales_subdir;

	echo 'Usage: ', "\n";
	echo ' ', $my_basename, ' [-b <app_root>] <CORE|CWD> [extract]', "\n";
	echo ' ', $my_basename, ' [-b <app_root>] <CORE|CWD> merge <locale> [locale..]', "\n";
	echo ' ', $my_basename, ' [-b <app_root>] <CORE|CWD> convert <locale> [locale..]', "\n\n";
	echo 'CORE: work on the core application.', "\n";
	echo 'CWD : work on current working directory (plugin mode).', "\n";
	echo "\n";
	echo 'By default, the translatable strings get extracted into ', $locales_subdir, 'messages.pot.', "\n";
	echo "\n";
	echo 'By adding \'merge <locale>\' to the command line arguments, you\'ll merge', "\n";
	echo 'the locale\'s messages.po file with the messages.pot file. This is useful', "\n";
	echo 'after having updated the messages.pot file, obviously.', "\n";
	echo "\n";
	echo 'By adding \'convert <locale>\' to the command line arguments, you\'ll convert', "\n";
	echo 'the locale\'s messages.po file to _global.php, which ', $app_name, ' uses.', "\n";
	echo "\n";
	echo 'E.g.:', "\n";
	echo ' php ', $my_basename, ' CORE', "\n";
	echo ' php ', $my_basename, ' CORE merge de_DE', "\n";
	echo ' ..edit .po file..', "\n";
	echo ' php ', $my_basename, ' CORE convert de_DE', "\n";
	echo "\n";
	echo 'This script expects the ', $app_name, ' root directory to be at [', $_app_root, '].', "\n";
	echo 'You can specify an alternative root directory using the -b option, e. g.:', "\n";
	echo ' php ', $my_basename, ' -b ../blogs CORE', "\n";
	echo '...but if you are reading this message, the root directory is set correctly.', "\n";
}


// Allow overriding of the application root directory via command line (-b option):
$opts = getopt( 'b:' );
if( isset( $opts['b'] ) )
{
	$_app_root = $opts['b'];

	// Silently modify $argc and $argv:
	$argc -= 2;
	// This will fail if somebody passes an argument with more than one
	// option, e. g. "-bar".
	array_splice( $argv, array_search( '-b', $argv ), 2 );

	//var_dump( $argc, $argv );
}

if( ! is_dir( $_app_root ) )
{
	echo 'Error: Application root directory [', $_app_root, '] does not exist!', "\n";
	echo 'You can specify an alternative root directory by using the -b option:', "\n";
	echo ' php ', $my_basename, ' -b /path/to/application/root/directory CORE', "\n";
	exit( 2 );
}

require_once $_app_root.'/qp_config/_config.php';

echo '** gettext helper tool for ', $app_name, ' **', "\n\n";

if( $argc < 2 )
{
	echo_usage();
	exit(1);
}

switch( strtoupper($argv[1]) )
{
	case 'CORE':
		echo 'CORE mode...', "\n";
		$mode = 'CORE';
		// Our root directory is the application root directory:
		$dir_root = $_app_root;
		break;

	case 'CWD':
		echo 'Using current working directory (plugin mode)...', "\n";
		$mode = 'CWD';
		$dir_root = getcwd();
		break;

	default:
		echo_usage();
		exit(1);
}

$action = 'extract';
if( isset($argv[2]) )
{
	$action = strtolower( $argv[2] );

	if( ! in_array( $action, array( 'extract', 'merge', 'convert' ) ) )
	{
		echo_usage();
		exit( 1 );
	}
	else if( $action == 'convert' || $action == 'merge' )
	{
		if( ! isset($argv[3]) )
		{
			echo_usage();
			exit(1);
		}

		$_locales = array_slice( $argv, 3 );
	}
}

// ---- COMMON CHECKS: ----

if( $action != 'convert' )
{
	// Check that all external tools are available:
	foreach( array( 'xgettext', 'msgmerge', 'find', 'xargs', 'sed' ) as $testtool )
	{
		exec( $testtool.' --version 2>/dev/null', $output, $return );
		if( $return == 127 )
		{	// Command not found (return code specified by POSIX):
			die( 'This script needs the '.$testtool.' tool.'."\n" );
		}
	}
}

if( ! is_dir($dir_root) )
{
	die( 'Fatal error: The path ['.$dir_root.'] was not found!'."\n" );
}
// Normalize path:
$dir_root = realpath($dir_root).'/';

// This is required for the cygwin on Windows:
$dir_root = str_replace('\\', '/', $dir_root);

// The messages.pot (template) file:
if( $mode == 'CORE' )
{
	$file_pot = $locales_path;
}
else
{	// Plugin mode:
	$file_pot = $dir_root.'po/';
}
$file_pot .= 'messages.pot';

if( $action == 'extract' )
{
	if( ! is_writable($file_pot) )
	{
		$locale_dir = dirname( $file_pot );

		if( ! file_exists( $locale_dir ) )
		{
			echo 'Directory [', $locale_dir, '] does not exist..', "\n";

			if( ! mkdir( $locale_dir ) )
			{
				die( 'FATAL: could not create directory ['.$locale_dir.']!'."\n" );
			}
			echo 'Created directory.', "\n";
		}

		if( ! file_exists( $file_pot ) )
		{
			touch( $file_pot );
		}


		if( ! is_writable($file_pot) )
		{
			die( 'FATAL: The file ['.$file_pot.'] is not writable.'."\n" );
		}
	}

	$dir_root_basename = basename($dir_root);
	if( isset($argv[3]) )
	{ // File(s) specified
		echo 'Extracting T_() and NT_() strings from given files below "';
		echo $dir_root_basename, '" into "', $file_pot, '"... ';
		$cmd = '';
	}
	else
	{
		echo 'Extracting T_() and NT_() strings from all .php files below "';
		echo $dir_root_basename, '" into "', $file_pot, '"... ';

		// Change working dir (to simplify exlude patterns below):
		chdir( $dir_root );

		/* Find *.php files, but exclude:
		 *  - all directories below qp_templates/
		 *  - the qp_plugins/ directory
		 *  - the build/ directory
		 */
		$cmd  = 'find . \\( ';
		$cmd .= '-path \'./qp_templates/*[!/]/*\' -o -path \'./qp_plugins/*\' -o -path \'./build/*\'';
		$cmd .= ' \\) -prune -o \\( -iname \'*.php\' -print0 \\) | xargs -0 ';
	}
	$cmd .= 'xgettext -o '.escapeshellarg($file_pot).' --from-code=iso-8859-15 --no-wrap --add-comments=TRANS --copyright-holder="The Quam Plures developers" --msgid-bugs-address=http://quamplures.net/ --keyword=T_ --keyword=NT_ --keyword=TS_ -F';

	// Append filenames, if specified:
	if( isset($argv[3]) )
	{
		$n = count($argv);
		for( $i = 3; $i < $n; $i++ )
		{
			$cmd .= ' '.escapeshellarg($argv[$i]);
		}
	}

	//echo 'Command: [', $cmd, ']', "\n";
	system( $cmd, $return_var );
	if( $return_var !== 0 )
	{
		die('Command failed (return code: '.$return_var.')!'."\n");
	}
	echo "[ok]\n";


	// Replace various things (see comments)
	echo 'Automagically search & replace in messages.pot.. ';
	$data = file_get_contents( $file_pot );

	$data = str_replace( "\r", '', $data );
	
	// Make paths relative:
	if( isset( $argv[3] ) )
	{	// File names given
		$rep_path = $dir_root;
	}
	else
	{	// Core mode (prepend root path to file names in POT file):
		$rep_path = '';
	}
	$data = preg_replace( '~^#: .*$~me', 'str_replace( \' '.$rep_path.'\', \' ../\', \'$0\' )', $data );

	file_put_contents( $file_pot, $data );

	if( $mode == 'CORE' )
	{ // Replace header "vars" in first 20 lines:
		system( 'sed -i "1,20{'
			.'s/PACKAGE/'.$app_name.'/;'
			.'s/VERSION/'.$app_version.'/;'
			.'s/# SOME DESCRIPTIVE TITLE./# '.$app_name.' - Language file/;'
			.'s/(C) YEAR/(C) 2009-'.date('Y').'/;'
			.'s/CHARSET/iso-8859-1/;'
			.'}" '.escapeshellarg($file_pot) );
	}
	else
	{
		// TODO: try to detect usage with a plugin and s&r accordingly
	}
	echo '[ok]', "\n";

	exit(0);
}


if( $action == 'merge' )
{ // Merge with existing .po files:
	if( ! @is_readable( $file_pot ) )
	{
		echo 'FATAL: [', $file_pot, '] is not readable!', "\n";
		exit(1);
	}

	foreach( $_locales as $l_locale )
	{
		if( $mode == 'CORE' )
		{
			$l_file_po = $locales_path;
		}
		else
		{	// Plugin mode:
			$l_file_po = $dir_root.'/po/';
		}
		$l_file_po .= $l_locale.'.po';

		echo 'Merging with '.$l_locale.'.. ';

		if( ! file_exists( $l_file_po ) )
		{
			echo 'PO file [', $l_file_po, '] not found!', "\n";
			continue;
		}

		system( 'msgmerge -U -F --no-wrap '.escapeshellarg($l_file_po).' '.escapeshellarg($file_pot) );
		# delete old TRANS comments and make automatic ones valid comments:
		system( 'sed -i -r "/^#\\s+TRANS:/d; s/^#\\. TRANS:/# TRANS:/;" '.$l_file_po );
		echo 'Written [', $l_file_po, '].', "\n";
		echo "\n";
	}

	exit(0);
}


if( $action == 'convert' )
{ // convert messages.PO files to _global.php
	require_once $_app_root.'/qp_inc/locales/_pofile.class.php';

	foreach( $_locales as $l_locale )
	{
		if( $mode == 'CORE' )
		{
			$l_file_po       = $locales_path;
			$global_file_dir = $locales_path;
		}
		else
		{	// Plugin mode...
			$l_file_po       = $dir_root.'po/';
			$global_file_dir = $dir_root.'po/';
		}

		$l_file_po       .= $l_locale.'.po';
		$global_file_dir .= 'php/'.$l_locale;
		$global_file_path = $global_file_dir.'/_global.php';

		echo 'Converting '.$l_locale.'.. ';

		if( !file_exists( $l_file_po ) )
		{
			echo 'PO file [', $l_file_po, '] not found!', "\n";
			continue;
		}
		else if( ! file_exists( $global_file_dir ) &&
				 ! mkdir( $global_file_dir, 0777, true ) )
		{
			echo 'Could not create directory [', $global_file_dir, ']!', "\n";
			continue;
		}

		$POFile = new POFile($l_file_po);
		$POFile->read(false);
		$r = $POFile->write_evo_trans($global_file_path, $l_locale);

		if( $r !== true )
		{
			echo 'Error: ', $r, "\n";
			continue;
		}

		echo '[ok]', "\n";
	}

	exit(0);
}
